#
#  Zope Internationalization Utilities
#
import string
import types


isFunctionType={}
for name in ['BuiltinFunctionType', 'BuiltinMethodType', 'ClassType',
             'FunctionType', 'LambdaType', 'MethodType', 'UnboundMethodType']:
    try: isFunctionType[ getattr( types, name ) ] = 1
    except: pass

try: # Add function and method types from Extension Classes
    import ExtensionClass
    isFunctionType[ExtensionClass.PythonMethodType]=1
    isFunctionType[ExtensionClass.ExtensionMethodType]=1
except: pass

isFunctionType=isFunctionType.has_key

def getPhysicalPath( self ):
    path = ( self.id(), )
    return self.aq_parent.getPhysicalPath() + path

def build_children( self ):
    children = {}
    for id, obj in self.objectItems() :
        children[ id ] = obj
    return children


def primary_version_name( self, REQUEST ):
    """
    """
    if hasattr( self, 'PRIMARY_VERSION_NAME' ):
       result = getattr( self, 'PRIMARY_VERSION_NAME' )
    else:
       result = self[ 'DEFAULT_PRIMARY_VERSION_NAME' ]
    if isFunctionType( type( result ) ):
       result = result()
    return result


def locale_cookie_name( self, REQUEST ):
    """
    """
    if hasattr( self, 'LOCALE_COOKIE_NAME' ):
       result = getattr( self, 'LOCALE_COOKIE_NAME' )
    else:
       result = self[ 'DEFAULT_LOCALE_COOKIE_NAME' ]
    if isFunctionType( type( result ) ):
       result = result()
    return result

def find_locale( self, REQUEST, children=None ):
    """ Determine the user's preferred locale and the one we can serve up.
        (i.e., the first matching child object):

        1. Check for cookie, "ZI18N_preferred_locale"
        2. Use environment, "HTTP_ACCEPT_LANGUAGE"
        3. Use acquired "ZI18N_default_locale"
        4. Return "index_html"

        Note that the preferred locale comes either from the cookie or from
        the HTTP environment variable.
    """
    if not children:
        children = build_children( self )

    cookie_name = locale_cookie_name( self, REQUEST )
    locale = REQUEST.cookies.get( cookie_name )

    if locale and children.has_key( locale ):
        return locale, locale

    preferred_locale = locale or None

    for locale in string.split( REQUEST[ 'HTTP_ACCEPT_LANGUAGE' ], ',' ):
        if not locale:
           break
        if not preferred_locale:
            preferred_locale = locale
        if locale and children.has_key( locale ):
            return preferred_locale, locale

    if hasattr( self, 'ZI18N_default_locale' ): 
        locale = self.ZI18N_default_locale
        if children.has_key( locale ):
            return preferred_locale, locale
    
    return preferred_locale, None


def localize( self, REQUEST ):
    """ Return locale-specific version.
    """
    children = build_children( self )

    preferred_locale, locale = find_locale( self, REQUEST, children )

    return locale and children[ locale ] or self


class Localize_request :

    def __init__( self, name, cookies, langs ):
        self.name                 = name
        self.cookies              = cookies
        self.HTTP_ACCEPT_LANGUAGE = langs
    
    def __getitem__( self, key ):
        return self.__dict__[ key ]

TEST_LOCALE_COOKIE = 'ZI18N_preferred_locale' 

class Localize_tester :

    def __init__( self, index, default_locale=None ) :
        self.objects    = {}
        self.name       = index
        if default_locale:
            self.ZI18N_default_locale = default_locale
        self.addObject( 'index_html', '<%s>' % index )
        self.DEFAULT_LOCALE_COOKIE_NAME = TEST_LOCALE_COOKIE

    def addObject( self, id, obj ):
        self.objects[ id ] = obj
    
    def objectItems( self ):
        result = []
        for id, obj in self.objects.items() :
            result.append( ( id, obj ) )
        return result

    def __getitem__( self, key ):
        return self.__dict__[ key ]


def test_localize( tester, request ):
    localized_version = localize( tester, request )
    if localized_version is tester:
        return tester.objects[ 'index_html' ]
    return localized_version

if __name__ == '__main__' :

    # Nothing
    req_nomatch     = Localize_request( 'req_nomatch'
                                      , {}
                                      , ''
                                      )

    # Has cookie for 'es'; HTTP_ACCEPT_LANGUAGE entries shouldn't match.
    req_cookie      = Localize_request( 'req_cookie'
                                      , { TEST_LOCALE_COOKIE : 'es' }
                                      , 'de,en'
                                      )

    # Has no cookie; HTTP_ACCEPT_LANGUAGE entries should match.
    req_environ     = Localize_request( 'req_environ'
                                      , {}
                                      , 'de,en'
                                      )
    
    requests = ( req_nomatch, req_cookie, req_environ )
    
    # Just index_html
    tester_bare     = Localize_tester( 'tester_bare' )

    # Has default_locale, and matching doc.
    tester_defloc   = Localize_tester( 'tester_defloc', 'fr' )
    tester_defloc.addObject( 'fr', '<tester_defloc_fr>' )

    # Has default_locale, but no matching doc.
    tester_defloc_x   = Localize_tester( 'tester_defloc_x', 'fr' )
    tester_defloc_x.addObject( 'de', '<tester_defloc_de>' )
    tester_defloc_x.addObject( 'es', '<tester_defloc_es>' )

    testers = ( tester_bare, tester_defloc, tester_defloc_x )


    for request in requests :
        print 'Request: %s' % request.name
        for tester in testers :
            print '  Tester: %-15s localized: %s' % \
            ( tester.name, test_localize( tester, request ) )
        print ''
